package fileChooser;

import java.io.*;
import org.w3c.dom.*;
import java.util.HashMap;
import java.util.logging.Level;
import java.util.logging.Logger;
import xmlUtil.*;
import graphLib.*;

/**
 * NetworkData.java
 * File IO for XML file
 * @author tadaki
 */
public class NetworkData extends NetworkFile {

    private Element root;
    private Document doc = null;
    private String docNS = "http://aoba.cc.saga-u.ac.jp/ns";
    private String docPrefix = null;
    private double s;

    /** Creates a new instance of NetworkData */
    public NetworkData() {
    }

    @Override
    /** Output network structure to files
     * @param network 出力するnetwork
     * @param outputFile 出力ファイル
     * @param vimap 頂点と個体の対応マップ
     * @param s リンク切り替え確率
     * @return 出力の正否
     * @throws IOException
     */
    public boolean output(Network network, String outputFile)
            throws IOException {
        String path = System.getProperty("user.dir");
        return output(network, outputFile, path);
    }

    @Override
    /** Output network structure to files
     * @param network 出力するnetwork
     * @param outputFile 出力ファイル
     * @param vimap 頂点と個体の対応マップ
     * @param s リンク切り替え確率
     * @param path xsdファイルを出力するパス
     * @return 出力の正否
     * @throws IOException
     */
    public boolean output(Network network, String outputFile, String path)
            throws IOException {
        boolean b = false;
        ClassLoader cl = this.getClass().getClassLoader();
        try (InputStream is =
                        cl.getResourceAsStream("fileChooser/template.xml")) {
            if (is == null) {
                return b;
            }

            outputXSD(path);//xsdの出力
            b = setDocument(is);//template読み込み
            if (!b) {
                return b;
            }
            clearXML();
            createXML(network);
            //出力
            XMLwriterToFolder writer = new XMLwriterToFolder(outputFile);
            writer.setDocument(doc);
            writer.putDomDocument();
        }
        return true;
    }

    @Override
    public boolean output(Network network, OutputStreamWriter swriter)
            throws IOException {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    /**
     * Output xsd file to the specified path
     * @param path path where the xsd file is stored
     */
    private void outputXSD(String path) throws IOException {
        ClassLoader cl = this.getClass().getClassLoader();
        InputStream is = cl.getResourceAsStream("networkData/Network.xsd");
        BufferedWriter fw;
        try (BufferedReader reader = new BufferedReader(
                        new InputStreamReader(is, enc))) {
            String xsdFile = path + File.separatorChar + "Network.xsd";
            fw = new BufferedWriter(
                    new OutputStreamWriter(
                    new FileOutputStream(xsdFile), enc));
            String line;
            while ((line = reader.readLine()) != null) {
                fw.write(line);
                fw.newLine();
            }
        }
        fw.close();
    }

    /**
     * Clear contents of XML file
     */
    private void clearXML() {
        root = doc.getDocumentElement();
        //頂点の削除
        Element vertexes =
                (Element) root.getElementsByTagNameNS(
                docNS, "Vertexes").item(0);
        NodeList children = vertexes.getChildNodes();
        while (children.getLength() > 0) {
            vertexes.removeChild(children.item(0));
        }
        //弧の削除
        Element arcs =
                (Element) root.getElementsByTagNameNS(
                docNS, "Arcs").item(0);
        children = arcs.getChildNodes();
        while (children.getLength() > 0) {
            arcs.removeChild(children.item(0));
        }
    }

    /**
     * Create document from the network
     * @param network
     */
    private void createXML(Network network) {
        root = doc.getDocumentElement();
        boolean directed = false;
        //Header
        Element header =
                (Element) root.getElementsByTagNameNS(docNS, "Header").
                item(0);
        docPrefix = header.getPrefix();
        header.setAttribute("name", network.toString());
        directed = network.isDirected();
        if (directed) {
            header.setAttribute("directed", "true");
        } else {
            header.setAttribute("directed", "false");
        }

        //vertexes
        Element vertexes =
                (Element) root.getElementsByTagNameNS(docNS, "Vertexes").
                item(0);
        for (Vertex v : network.getVertexes()) {//各頂点
            vertexes.appendChild(setNode(v));
        }

        //Arcs
        Element arcs =
                (Element) root.getElementsByTagNameNS(docNS, "Arcs").item(0);
        for (Vertex v : network.getVertexes()) {
            for (Arc a : network.getArcs(v)) {
                arcs.appendChild(setArc(a, v, network));
            }
        }
    }

    /**
     * 頂点データを対応するタグに設定
     * @param v 頂点データ
     * @return  タグ
     */
    private Element setNode(Vertex v) {
        Element node = doc.createElementNS(docNS, "Vertex");
        node.setPrefix(docPrefix);
        node.setAttribute("name", v.toString());
        java.awt.geom.Point2D.Double p = v.getPoint();
        if (p != null) {
            Element point = doc.createElementNS(docNS, "Point");
            point.setPrefix(docPrefix);
            point.setAttribute("x", String.valueOf(p.x));
            point.setAttribute("y", String.valueOf(p.y));
            node.appendChild(point);
        }
        return node;
    }

    private void setVertex(Network network, Element node,
            HashMap<String, Vertex> map) {
        Vertex v = new Vertex(node.getAttribute("name"));
        NodeList pl = node.getElementsByTagNameNS(docNS, "Point");
        if (pl != null && pl.getLength() > 0) {
            Element p = (Element) pl.item(0);
            v.setPoint(Double.valueOf(p.getAttribute("x")),
                    Double.valueOf(p.getAttribute("y")));
        }
        network.addVertex(v);
        map.put(v.toString(), v);

    }

    /**
     * 弧のデータを対応するタグに設定
     * @param a 弧
     * @param v 始点の頂点
     * @return  タグ
     */
    private Element setArc(Arc a, Vertex v, Network network) {
        Element arc = doc.createElementNS(docNS, "Arc");
        arc.setPrefix(docPrefix);
        Vertex w = network.getTerminal(a, v);
        arc.setAttribute("from", v.toString());
        arc.setAttribute("to", w.toString());
        arc.setAttribute("weight", String.valueOf(a.getWeight()));
        return arc;
    }

    /**
     * タグから弧のデータを取得してネットワークに設定
     * @param network
     * @param arcNode
     * @param map
     */
    private void getArc(Network network, Element arcNode,
            HashMap<String, Vertex> map) {
        String vs = arcNode.getAttribute("from");
        String ws = arcNode.getAttribute("to");
        String weightString = arcNode.getAttribute("weight");
        String name = arcNode.getAttribute("name");
        Arc a = network.addArc(map.get(vs), map.get(ws), name);
        if (weightString != null) {
            a.setWeight(Double.valueOf(weightString));
        }
    }

    @Override
    /** Constructing network from files
     * @param filename
     * @return
     */
    public Network input(String filename) throws IOException {
        File file = new File(filename);
        return input(file);
    }

    @Override
    public Network input(File file) throws IOException {
        boolean b = true;
        Network network = null;
        FileInputStream is = null;
        is = new FileInputStream(file);
        if (is == null) {
            Logger.getLogger(NetworkData.class.getName()).
                    log(Level.SEVERE, "Can not open {0}", file.getName());
            return null;
        }
        b = setDocument(is);
        network = setData();
        return network;
    }

    @Override
    public Network input(InputStreamReader sreader) throws IOException {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    private Network setData() {
        Network network = null;

        root = doc.getDocumentElement();
        //Header
        Element header =
                (Element) root.getElementsByTagNameNS(docNS, "Header").item(0);
        network = new Network(header.getAttribute("name"));
        boolean directed = false;
        if (header.getAttribute("directed").equals("true")) {
            directed = true;
        }
        network.setDirected(directed);
        //vertexes
        NodeList vertexes = root.getElementsByTagNameNS(docNS, "Vertex");
        HashMap<String, Vertex> map = new HashMap<>();
        int n = vertexes.getLength();
        for (int i = 0; i < n; i++) {
            setVertex(network, (Element) vertexes.item(i), map);
        }

        //Arcs
        NodeList arcs = root.getElementsByTagNameNS(docNS, "Arc");
        int m = arcs.getLength();
        for (int i = 0; i < m; i++) {
            getArc(network, (Element) arcs.item(i), map);
        }
        return network;
    }

    /**
     * Read document from Input stream
     * @param is inputstream
     * @return succeed or fail
     */
    private boolean setDocument(InputStream is) {
        boolean b = true;
        doc = DOMUtil.parse(is);
        return b;
    }
}
