package graphDraw;

import java.awt.geom.Point2D;
import java.awt.geom.Line2D;
import java.awt.Color;
import java.awt.Graphics2D;

/**
 * 弧を描くクラス
 * 並列弧や孤立弧に対応できない
 * @author tadaki
 */
public class Arrow {

    /** 終点の座標 */
    private Point2D.Double head = null;
    /** 始点の座標 */
    private Point2D.Double tail = null;
    /** 頂点の半径 */
    private double r = 0.;
    /** 始点から頂点を見た角度 */
    private double angle = 0.;
    /** 終点の矢印の大きさ */
    private double s = 10.;
    /** ラベル */
    private String name = null;
    private boolean directed = true;
    private Color color = Color.BLACK;

    /**
     * Creates a new instance of Arrow
     * @param tail 始点
     * @param head 終点
     * @param r 頂点の半径
     * @param name ラベル
     */
    public Arrow(Point2D.Double tail, Point2D.Double head,
            double r, String name) {
        this(tail, head, r, name, true);
    }

    /**
     * Creates a new instance of Arrow
     * @param tail 始点
     * @param head 終点
     * @param r 頂点の半径
     * @param name ラベル
     * @param b 有向・無向
     */
    public Arrow(Point2D.Double tail, Point2D.Double head,
            double r, String name, boolean b) {
        this.head = new Point2D.Double(head.x, head.y);
        this.tail = new Point2D.Double(tail.x, tail.y);
        this.r = r;
        this.name = name;
        directed=b;
    }

    /**
     * 弧を描く
     * @param g 使用するGraphicsオブジェクト
     */
    public void draw(Graphics2D g) {
        g.setColor(getColor());
        getAngle();
        Point2D.Double t = new Point2D.Double(
                tail.x + r * Math.cos(angle),
                tail.y + r * Math.sin(angle));
        Point2D.Double h = new Point2D.Double(
                head.x - r * Math.cos(angle),
                head.y - r * Math.sin(angle));
        g.draw(new Line2D.Double(t, h));

        if (directed) {
            Point2D.Double hu = new Point2D.Double(
                    h.x + s * Math.cos(Math.PI + angle - Math.PI / 6.),
                    h.y + s * Math.sin(Math.PI + angle - Math.PI / 6.));
            g.draw(new Line2D.Double(h, hu));
            Point2D.Double hd = new Point2D.Double(
                    h.x + s * Math.cos(Math.PI + angle + Math.PI / 6.),
                    h.y + s * Math.sin(Math.PI + angle + Math.PI / 6.));
            g.draw(new java.awt.geom.Line2D.Double(h, hd));
        }
        Point2D.Double c = new Point2D.Double(
                (head.x + tail.x) / 2., (head.y + tail.y) / 2.);
        g.drawString(name, (float) c.x, (float) c.y);
    }
    /** 設定及び取得メソッド ***********************************************/
    /**
     * 弧の終点を設定
     * @param head 弧の終点
     */
    public void setHead(Point2D.Double head) {
        this.head = new Point2D.Double(head.x, head.y);
    }

    /**
     * 弧の始点を設定
     * @param tail 弧の始点
     */
    public void setTail(Point2D.Double tail) {
        this.tail = new Point2D.Double(tail.x, tail.y);
    }

    /**
     * 頂点の半径を設定
     * @param r 頂点の半径
     */
    public void setR(double r) {
        this.r = r;
    }

    /**
     * 始点から終点への角度を求める
     * @return 角度
     */
    public double getAngle() {
        double dx = head.x - tail.x;
        double dy = head.y - tail.y;
        angle = Math.atan2(dy, dx);
        return angle;
    }

    /**
     * 矢印の大きさを設定する
     * @param s 矢印の大きさ
     */
    public void setS(double s) {
        this.s = s;
    }

    public boolean isDirected() {
        return directed;
    }

    public void setDirected(boolean directed) {
        this.directed = directed;
    }

    /**
     * @return the color
     */
    public Color getColor() {
        return color;
    }

    /**
     * @param color the color to set
     */
    public void setColor(Color color) {
        this.color = color;
    }
}
