package graphLib;

import java.util.List;
import java.util.HashMap;
import java.io.*;

/**
 * グラフのクラス
 * 頂点や弧が連結の情報を持たないように変更(2011/4/6)
 * @author tadaki
 */
public class Graph extends GraphBase {

    /** 隣接行列 */
    protected int adjacent[][] = null;
    /** 二つの頂点の間に道があるかの有無 */
    protected boolean connectedness[][] = null;
    private HashMap<Vertex, Vertex> map = null;
    private HashMap<Arc, Arc> amap = null;

    /**
     * Creates a new instance of Graph
     * @param name グラフのラベル
     */
    public Graph(String name) {
        this.name = name;
        vertexes = Utils.createVertexList();
        arcs = Utils.createArcList();
        a2vHead = new HashMap<>();
        a2vTail = new HashMap<>();
        v2a = new HashMap<>();
    }

    /**
     * 弧を削除
     * @param a 削除対象の弧
     * @return 削除成功
     */
    public boolean removeArc(Arc a) {
        boolean b = true;
        if (a2vHead.containsKey(a)) {
            Vertex head = a2vHead.remove(a);
            List<Arc> list = v2a.get(head);
            b = b & list.remove(a);
        } else {
            b = false;
        }
        if (a2vTail.containsKey(a)) {
            Vertex tail = a2vTail.remove(a);
            b = b & true;
            if (!directed) {
                List<Arc> list = v2a.get(tail);
                b = b & list.remove(a);
            }
        } else {
            b = b & false;
        }
        b = b & arcs.remove(a);
        return b;
    }

    /**
     * 二つの頂点を結ぶ弧を削除
     * @param from 削除対象の弧の始点
     * @param to   削除対象の弧の終点
     * @return 削除成功
     */
    public boolean removeArc(Vertex from, Vertex to) {
        Arc a = getArc(from, to);
        if (a == null) {
            return false;
        }
        return removeArc(a);
    }

    /**
     * 指定した弧の存在の有無
     * @param v 調べる弧
     * @return 有無
     */
    public boolean contains(Vertex v) {
        return vertexes.contains(v);
    }

    /**
     * 指定した弧の存在の有無
     * @param a 調べる弧
     * @return 有無
     */
    public boolean contains(Arc a) {
        return arcs.contains(a);
    }

    /**
     * 隣接行列を生成する
     * @return 隣接行列
     */
    public int[][] getAdjacent() {
        int n = vertexes.size();
        adjacent = new int[n][];
        for (int i = 0; i < n; i++) {
            adjacent[i] = new int[n];
            for (int j = 0; j < n; j++) {
                adjacent[i][j] = 0;
            }
        }
        if (directed) {
            for (int i = 0; i < n; i++) {
                Vertex v = vertexes.get(i);
                for (Arc a : v2a.get(v)) {
                    Vertex t = a2vTail.get(a);
                    int l = vertexes.indexOf(t);
                    adjacent[l][i]++;
                }
            }
        } else {
            for (int i = 0; i < n; i++) {
                Vertex v = vertexes.get(i);
                for (Arc a : v2a.get(v)) {
                    Vertex t = a2vTail.get(a);
                    if (!t.equals(v)) {
                        int l = vertexes.indexOf(t);
                        adjacent[i][l]++;
                        adjacent[l][i]++;
                    }
                }
            }

        }
        checkConnectedness();
        return adjacent;
    }

    private void checkConnectedness() {
        int n = vertexes.size();
        connectedness = new boolean[n][];
        for (int i = 0; i < n; i++) {
            connectedness[i] = new boolean[n];
            for (int j = 0; j < n; j++) {
                connectedness[i][j] = (adjacent[i][j] > 0);
            }
        }
        for (int t = 0; t < n; t++) {
            boolean b[][] = new boolean[n][];
            for (int i = 0; i < n; i++) {
                b[i] = new boolean[n];
                for (int j = 0; j < n; j++) {
                    b[i][j] = false;
                }
            }
            for (int i = 0; i < n; i++) {
                for (int j = 0; j < n; j++) {
                    b[i][j] = false;
                    for (int k = 0; k < n; k++) {
                        b[i][j] = b[i][j]
                                | (connectedness[i][k] & (adjacent[k][j] > 0));
                    }
                }
            }
            for (int i = 0; i < n; i++) {
                System.arraycopy(b[i], 0, connectedness[i], 0, n);
            }
        }
    }

    /**
     * 二つの頂点を結ぶ道が存在するか
     * @param i 始点を指定する番号
     * @param j 終点を指定する番号
     * @return  道が存在すれば真
     */
    public boolean isConnected(int i, int j) {
        return connectedness[j][i];
    }

    /**
     * 二つの弧を結ぶ道が存在するか
     * @param s 始点
     * @param d 終点
     * @return  道が存在すれば真
     */
    public boolean isConnected(Vertex s, Vertex d) {
        int i = -1;
        for (int k = 0; k < vertexes.size(); k++) {
            if (vertexes.get(k).equals(s)) {
                i = k;
            }
        }
        if (i < 0) {
            System.err.println("Fail to find the vertex " + s.toString());
            return false;
        }
        int j = -1;
        for (int k = 0; k < vertexes.size(); k++) {
            if (vertexes.get(k).equals(d)) {
                j = k;
            }
        }
        if (j < 0) {
            System.err.println("Fail to find the vertex " + d.toString());
            return false;
        }
        return isConnected(i, j);
    }

    /**
     * 弧の一覧の設定
     * @param arcs  設定する弧の一覧
     */
    public void setArcs(List<Arc> arcs) {
        this.arcs = arcs;
    }

    /**
     * 頂点間を結ぶ道の有無を表す配列の取得
     * @return
     */
    public boolean[][] getConnectedness() {
        return connectedness;
    }

    /**
     * 頂点間を結ぶ道の有無を表す配列の設定     *
     * @param connectedness 配列
     */
    public void setConnectedness(boolean[][] connectedness) {
        this.connectedness = connectedness;
    }

    /** ファイル操作 **************************************************/
    /**
     * XMLファイルからグラフを読む
     * @param filename XMLファイル名
     * @return 読み出したグラフ
     * @throws IOException 入力時の例外
     */
    static public Graph readXMLFIle(String filename) throws IOException {
        return readXMLFile(new File(filename));
    }

    /**
     * XMLファイルからグラフを読む
     * @param file XMLファイル
     * @return 読み出したグラフ
     * @throws IOException 入力時の例外
     */
    static public Graph readXMLFile(File file) throws IOException {
        return fileChooser.FileUtil.openXML(file);
    }

    /**
     * グラフをXMLファイルへ保存
     * @param filename XMLファイル名
     * @throws IOException 書き込み時の例外
     */
    public void saveXMLFile(String filename) throws IOException {
        saveXMLFile(new File(filename));
    }

    /**
     * グラフをXMLファイルへ保存
     * @param file XMLファイル
     * @throws IOException 書き込み時の例外
     */
    public void saveXMLFile(File file) throws IOException {
        fileChooser.FileUtil.saveXML(file, (Network) this);
    }

    /********************************************************************/
    /**
     * コピーコンストラクタ
     * @param graph 元となるグラフ
     * @param copyArcs 弧もコピーするか
     */
    public Graph(final Graph graph) {
        this(graph, true);
    }

    public Graph(final Graph graph, boolean copyArcs) {
        this(graph.getName());
        directed = graph.isDirected();
        /** 頂点のコピー  */
        map = createVertexMap(graph.getVertexes());
        if (copyArcs) {
            /** 弧のコピー  */
            amap = createArcMap(graph.getArcs());
            for (Arc a : amap.keySet()) {
                arcs.add(amap.get(a));
            }
            /** 弧から頂点への写像のコピー */
            copyA2V(graph);
            /** 頂点を始点とする弧の一覧のコピー */
            copyV2A(graph);
        }
    }

    /**
     * 弧から頂点への写像のコピ
     * @param original 元となるグラフ
     */
    public final void copyA2V(final Graph original) {
        a2vHead = new HashMap<>();
        a2vTail = new HashMap<>();
        for (Arc a : amap.keySet()) {
            Arc aa = amap.get(a);
            Vertex head = map.get(original.a2vHead.get(a));
            Vertex tail = map.get(original.a2vTail.get(a));
            a2vHead.put(aa, head);
            a2vTail.put(aa, tail);
        }
    }

    public final void copyV2A(final Graph original) {
        v2a = new HashMap<>();
        for (Vertex v : map.keySet()) {
            Vertex w = map.get(v);
            List<Arc> aListNew = Utils.createArcList();
            List<Arc> aList = original.v2a.get(v);
            if (aList != null) {
                for (Arc a : aList) {
                    aListNew.add(amap.get(a));
                }
                v2a.put(w, aListNew);
            }
        }
    }

    private HashMap<Vertex, Vertex> createVertexMap(List<Vertex> vList) {
        HashMap<Vertex, Vertex> m = new HashMap<>();
        for (Vertex v : vList) {
            Vertex vn = new Vertex(v);
            addVertex(vn);
            m.put(v, vn);
        }
        return m;
    }

    private HashMap<Arc, Arc> createArcMap(List<Arc> aList) {
        HashMap<Arc, Arc> m = new HashMap<>();
        for (Arc a : aList) {
            Arc b = new Arc(a);
            m.put(a, b);
        }
        return m;
    }

    public HashMap<Arc, Arc> getAmap() {
        return amap;
    }

    public HashMap<Vertex, Vertex> getMap() {
        return map;
    }
}
